@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14/Segment_Delaunay_graph_Linf_2/include/CGAL/Segment_Delaunay_graph_Linf_2/Segment_Delaunay_graph_Linf_hierarchy_2_impl.h $
+// $Id: Segment_Delaunay_graph_Linf_hierarchy_2_impl.h ee57fc2 %aI Sébastien Loriot
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Panagiotis Cheilaris, Sandeep Kumar Dey, Evanthia Papadopoulou
@@ -359,7 +360,7 @@ insert_segment(const Point_2& p0, const Point_2& p1,
   Vertex_handle vertex;
 
   if ( hierarchy[0]->number_of_vertices() == 2 ) {
-    static Segments_in_hierarchy_tag stag;
+    Segments_in_hierarchy_tag stag;
 
     vertex = hierarchy[0]->insert_third(ss, vertices0[0], vertices1[0]);
     insert_segment_in_upper_levels(t, vertex->storage_site(),
@@ -389,8 +390,8 @@ insert_segment_interior(const Site_2& t, const Storage_site_2& ss,
   // arrangement_type
 
   // the tags
-  static Intersections_tag          itag;
-  static Segments_in_hierarchy_tag  stag;
+  Intersections_tag          itag;
+  Segments_in_hierarchy_tag  stag;
 
   CGAL_SDG_DEBUG(std::cout << "debug insert_segment_interior t=" << t 
                  << " at level " << level << std::endl;);
@@ -1010,6 +1011,9 @@ is_valid(bool verbose, int level) const
   bool result(true);
 
 #define DEBUGVALIDHIER true
+
+#include <CGAL/license/Segment_Delaunay_graph_Linf_2.h>
+
 //#undef DEBUGVALIDHIER
 
 #ifdef DEBUGVALIDHIER  
